@@ -53,487 +53,431 @@ struct button buttons[NKEYS] = {
 {
     KEY_0,
     "0",
-    do_number,
     NUMBER
 },
 {
     KEY_1,
     "1",
-    do_number,
     NUMBER
 },
 {
     KEY_2,
     "2",
-    do_number,
     NUMBER
 },    
 {     
     KEY_3,
     "3",
-    do_number,
     NUMBER
 },
 {
     KEY_4,
     "4",
-    do_number,
     NUMBER
 },
 {
     KEY_5,
     "5",
-    do_number,
     NUMBER
 },
 {
     KEY_6,
     "6",
-    do_number,
     NUMBER
 },
 {
     KEY_7,
     "7",
-    do_number,
     NUMBER
 },
 {
     KEY_8,
     "8",
-    do_number,
     NUMBER
 },
 {
     KEY_9,
     "9",
-    do_number,
     NUMBER
 },
 {
     KEY_A,
     "A",
-    do_number,
     NUMBER
 },
 {
     KEY_B,
     "B",
-    do_number,
     NUMBER
 },    
 {     
     KEY_C,
     "C",
-    do_number,
     NUMBER
 },
 {
     KEY_D,
     "D",
-    do_number,
     NUMBER
 },
 {
     KEY_E,
     "E",
-    do_number,
     NUMBER
 },
 {
     KEY_F,
     "F",
-    do_number,
     NUMBER
 },
 {    
     KEY_NUMERIC_POINT,
     ".",
-    do_point,
     NUMBER
 },
 {
     KEY_CALCULATE,
     NULL,
-    do_calc,
     0
 },
 {
     KEY_CLEAR,
     NULL,
-    do_clear,
     0
 },
 {
     KEY_CLEAR_ENTRY,
     NULL,
-    do_clear_entry,
     0
 },
 {
     KEY_START_BLOCK,
     "(",
-    do_paren,
     0
 },
 {
     KEY_END_BLOCK,
     ")",
-    do_paren,
     0
 },
 {
     KEY_ADD,
     "+",
-    do_calc,
     0
 },
 {
     KEY_SUBTRACT,
     "-",
-    do_calc,
     0
 },
 {
     KEY_MULTIPLY,
     "*",
-    do_calc,
     0
 },
 {
     KEY_DIVIDE,
     "/",
-    do_calc,
     0
 },
 {
     KEY_BACKSPACE,
     NULL,
-    do_backspace,
     0
 },
 {
     KEY_DELETE,
     NULL,
-    do_delete,
     0
 },
 {
     KEY_CHANGE_SIGN,
     NULL,
-    do_immed, 
     0
 },
 {
     KEY_INTEGER,
     "Int",
-    do_portion,
     FUNC
 },
 {
     KEY_FRACTION,
     "Frac",
-    do_portion,
     FUNC  
 },
 {
     KEY_PERCENTAGE,
     "%",
-    do_percent,  
     0
 },
 {
     KEY_SQUARE,
     "^2",
-    do_immed,  
     POSTFIXOP
 },
 {
     KEY_SQUARE_ROOT,
     "Sqrt",
-    do_immed, 
     FUNC
 },
 {
     KEY_RECIPROCAL,
     NULL,
-    do_immed, 
     0
 },
 {
     KEY_E_POW_X,
     "e^",
-    do_immed, 
     PREFIXOP
 },
 {
     KEY_10_POW_X,
     "10^",
-    do_immed, 
     PREFIXOP
 },       
 {
     KEY_X_POW_Y,
     "^",
-    do_calc,
     POSTFIXOP
 },
 {
     KEY_FACTORIAL,
     "!",
-    do_immed, 
     POSTFIXOP
 },
 {
     KEY_RANDOM,
     "Rand",
-    do_immed, 
     0
 },
 {
     KEY_SIN,
     "Sin",
-    do_sin,
     FUNC
 },
 {
     KEY_SINH,
     "Sinh",
-    do_sinh,
     FUNC
 },
 {
     KEY_ASIN,
     "Asin",
-    do_asin,
     FUNC
 },
 {
     KEY_ASINH,
     "Asinh",
-    do_asinh,
     FUNC
 },
 {
     KEY_COS,
     "Cos",
-    do_cos,
     FUNC
 },
 {
     KEY_COSH,
     "Cosh",
-    do_cosh,
     FUNC
 },
 {
     KEY_ACOS,
     "Acos",
-    do_acos,
     FUNC
 },
 {
     KEY_ACOSH,
     "Acosh",
-    do_acosh,
     FUNC
 },
 {
     KEY_TAN,
     "Tan",
-    do_tan,
     FUNC
 },
 {
     KEY_TANH,
     "Tanh",
-    do_tanh,
     FUNC
 },
 {
     KEY_ATAN,
     "Atan",
-    do_atan,
     FUNC
 },
 {
     KEY_TAN,
     "Atanh",
-    do_atanh,
     FUNC
 },
 {
     KEY_NATURAL_LOGARITHM,
     "Ln",
-    do_immed, 
     FUNC
 },
 {
     KEY_LOGARITHM,
     "Log",
-    do_immed, 
     FUNC
 },
 {
     KEY_LOGARITHM2,
     "Log2",
-    do_immed, 
     FUNC
 },
 {
     KEY_ABSOLUTE_VALUE,
     "Abs",
-    do_portion,
     FUNC
 },
 {
     KEY_MASK_16,
     "u16",
-    do_immed,  
     FUNC
 },            
 {
     KEY_MASK_32,
     "u32",
-    do_immed,  
     FUNC
 },
 {
     KEY_MODULUS_DIVIDE,
     " Mod ",
-    do_calc,
     0
 },
 {
     KEY_EXPONENTIAL,
     "e",
-    do_expno,
     0
 },
 {
     KEY_NOT,
     "~",
-    do_immed, 
     0
 },
 {
     KEY_OR,
     " OR ",
-    do_calc,
     0
 },
 {
     KEY_AND,
     " AND ",
-    do_calc,
     0
 },       
 {
     KEY_XOR,
     " XOR ",
-    do_calc,
     0
 },
 {
     KEY_XNOR,
     " XNOR ",
-    do_calc,
     0
 },
 {
     KEY_FINC_CTRM,
     "Ctrm",
-    do_business,
     0
 },
 {
     KEY_FINC_DDB,
     "Ddb",
-    do_business,
     0
 },
 {
     KEY_FINC_FV,
     "Fv",
-    do_business,
+    0
+},
+{
+    KEY_FINC_GPM,
+    "Gpm",
     0
 },
 {
     KEY_FINC_PMT,
     "Pmt",
-    do_business,
     0
 },
 {
     KEY_FINC_PV,
     "Pv",
-    do_business,
     0
 },
 {
     KEY_FINC_RATE,
     "Rate",
-    do_business,
     0
 },
 {
     KEY_FINC_SLN,
     "Sln",
-    do_business,
     0
 },
 {
     KEY_FINC_SYD,
     "Syd",
-    do_business,
     0
 },
 {
     KEY_FINC_TERM,
     "Term",
-    do_business,
     0
 },
 {
     KEY_SHIFT,
     NULL,
-    do_shift,
     0
 },
 {
     KEY_STORE,
     NULL,
-    do_sto,
     0
 },
 {
     KEY_RECALL,
     NULL,
-    do_rcl,
     0
 },
 {
     KEY_EXCHANGE,
     NULL,
-    do_exchange,
     0
 },
 {
     KEY_SET_ACCURACY,
     NULL,
-    do_accuracy,
+    0
+},
+{
+    KEY_SET_BASE,
+    NULL,
+    0
+},
+{
+    KEY_SET_NUMBERTYPE,
+    NULL,
+    0
+},
+{
+    KEY_UNDO,
+    NULL,
+    0
+},
+{
+    KEY_REDO,
+    NULL,
     0
 },
 {
     KEY_CONSTANT,
     NULL,
-    do_constant,
     0
 },
 {
     KEY_FUNCTION,
     NULL,
-    do_function,
     0
 },
 };
@@ -544,43 +488,9 @@ struct button buttons[NKEYS] = {
 void
 doerr(char *errmes)
 {
-    switch (v->syntax) {
-        case NPA:
-            ui_set_error_state(TRUE);
-            display_set_string(errmes);
-            ui_beep();
-            break;
-
-        case EXPRS:
-            v->math_error = -MPMATH_ERR;
-            break;
-        
-        default:
-            assert(FALSE);
-    }
+    v->math_error = -MPMATH_ERR;
 }
 
-static void
-init_text()         /* Setup constant strings. */
-{
-    STRNCPY(v->con_names[0], _("Kilometer-to-mile conversion factor"),
-            MAXLINE - 1);
-    STRNCPY(v->con_names[1], _("square root of 2"), MAXLINE - 1);
-    STRNCPY(v->con_names[2], _("Euler's Number (e)"), MAXLINE - 1);
-    STRNCPY(v->con_names[3], _("pi"), MAXLINE - 1);
-    STRNCPY(v->con_names[4], _("Centimeter-to-inch conversion factor"),
-            MAXLINE - 1);
-    STRNCPY(v->con_names[5], _("degrees in a radian"), MAXLINE - 1);
-    STRNCPY(v->con_names[6], _("2 ^ 20"), MAXLINE - 1);
-    STRNCPY(v->con_names[7], _("Gram-to-ounce conversion factor"), MAXLINE - 1);
-    STRNCPY(v->con_names[8], 
-           _("Kilojoule-to-British-thermal-unit conversion factor"),
-            MAXLINE - 1);
-    STRNCPY(v->con_names[9], 
-           _("Cubic-centimeter-to-cubic-inch conversion factor"), MAXLINE - 1);
-}
-
-
 /* Default math library exception handling routine. */
 
 /*ARGSUSED*/
@@ -588,6 +498,7 @@ int
 matherr(exc)
 struct exception *exc;
 {
+    // FIXME: Useless string
     doerr(_("Error"));
 
     return(1);
@@ -599,7 +510,8 @@ getparam(char *s, char *argv[], char *errmes)
 {
     if (*argv != NULL && argv[0][0] != '-') {
         STRNCPY(s, *argv, MAXLINE - 1);
-    } else { 
+    } else {
+        // FIXME: Useless string
         /* Translators: the following string contains two strings that
          * are passed to it: the first is the gcalctool program name and
          * the second is an error message (see the last parameter in the 
@@ -614,13 +526,13 @@ getparam(char *s, char *argv[], char *errmes)
 void
 usage(char *progname)
 {
-    /* Translators: the following string contains two strings that
-     * are passed to it: the first is the gcalctool program name and
-     * the second is the program version number.
-     */
-    FPRINTF(stderr, _("%s version %s\n\n"), progname, VERSION);
-    FPRINTF(stderr, _("Usage: %s: [-E] [-u] [-a accuracy] "), progname);
-    FPRINTF(stderr, _("\t\t [-?] [-v] [-h]\n"));
+    /* Translators: This message is displayed on the command line when
+       help is requested. %1$s and $3$s are replaced with the name
+       of the program and %2$s with the version string */
+    FPRINTF(stderr, _("%1$s version %2$s\n"
+                      "\n"
+                      "Usage: %3$s: [-E] [-u] [-a accuracy] [-?] [-v] [-h]\n"),
+            progname, VERSION, progname);
     exit(1);
 }
 
@@ -641,11 +553,14 @@ get_options(int argc, char *argv[])      /* Extract command line options. */
 
                 case 'a' : 
                     INC;
+                    // FIXME: Useless string
                     getparam(next, argv, _("-a needs accuracy value"));
                     v->accuracy = atoi(next);
                     if (v->accuracy < 0 || v->accuracy > MAXACC) {
-                        FPRINTF(stderr, 
-                                _("%s: accuracy should be in the range 0-%d\n"),
+                        /* Translators: This message is displayed when an invalid accuracy value is passed
+                           as a command line argument. %1$s is replaced with the name of the program and %2$d
+                           is replaced with the maximum supported accuracy value */
+                        FPRINTF(stderr, _("%1$s: accuracy should be in the range 0-%2$d\n"),
                                 v->progname, MAXACC);
                         v->accuracy = DEFAULT_ACCURACY;
                     }
@@ -670,11 +585,13 @@ get_options(int argc, char *argv[])      /* Extract command line options. */
 
 
 static void
-init_constant(int n, gchar *value)
+init_constant(int n, const gchar *name, const gchar *value)
 {
     gchar *str = g_strdup(value);
 
-    MPstr_to_num(str, DEC, v->MPcon_vals[n]);
+    STRNCPY(v->con_names[n], name, MAXLINE - 1);
+
+    MPstr_to_num(str, 10, v->MPcon_vals[n]);
     g_free(str);
 }
 
@@ -693,24 +610,30 @@ init_state(void)
     v->MPerrors      = FALSE;               /* No error information. */
     acc              = MAX_DIGITS + 12;     /* MP internal accuracy. */
     size             = MP_SIZE;
-    mpset(&acc, &size, &size);
+    mpset(acc, size, size);
 
     v->error       = 0;            /* No calculator error initially. */    
-    v->current    = KEY_CALCULATE;
 
-    v->ltr.key_exp    = 0;         /* Not entering an exponent number. */    
-    v->ltr.noparens   = 0;         /* No unmatched brackets initially. */
-
-    init_constant(0, "0.621");                 /* kms/hr <=> miles/hr. */
-    init_constant(1, "1.4142135623");          /* square root of 2 */
-    init_constant(2, "2.7182818284");          /* e */
-    init_constant(3, "3.1415926536");          /* pi */
-    init_constant(4, "0.3937007");             /* cms <=> inch. */
-    init_constant(5, "57.295779513");          /* degrees/radian. */
-    init_constant(6, "1048576.0");             /* 2 ^ 20. */
-    init_constant(7, "0.0353");                /* grams <=> ounce. */
-    init_constant(8, "0.948");                 /* Kjoules <=> BTU's. */
-    init_constant(9, "0.0610");                /* cms3 <=> inches3. */
+    /* Translators: This is the label for the default constant, the number of miles in one kilometer (0.621) */
+    init_constant(0, _("Kilometer-to-mile conversion factor"), "0.621");
+    /* Translators: This is the label for the default constant, the square root of 2 (1.41421) */
+    init_constant(1, _("square root of 2"), "1.4142135623");
+    /* Translators: This is the label for the default constant, Euler's number (2.71828) */
+    init_constant(2, _("Euler's Number (e)"), "2.7182818284");
+    /* Translators: This is the label for the default constant, π (3.14159) */
+    init_constant(3, _("π"), "3.1415926536");
+    /* Translators: This is the label for the default constant, the number of inches in a centimeter (0.39370) */
+    init_constant(4, _("Centimeter-to-inch conversion factor"), "0.3937007");
+    /* Translators: This is the label for the default constant, the number of degrees in a radian (57.2958) */
+    init_constant(5, _("degrees in a radian"), "57.295779513");
+    /* Translators: This is the label for the default constant, 2 to the power of 20 (1048576) */
+    init_constant(6, _("2 ^ 20"), "1048576.0");
+    /* Translators: This is the label for the default constant, the number of ounces in one gram (0.0353) */
+    init_constant(7, _("Gram-to-ounce conversion factor"), "0.0353");
+    /* Translators: This is the label for the default constant, the number of British Thermal Units in one Kilojoule (0.948) */
+    init_constant(8, _("Kilojoule-to-British-thermal-unit conversion factor"), "0.948");
+    /* Translators: This is the label for the default constant, the number of cubic inches in one cubic centimeter (0.0610) */
+    init_constant(9, _("Cubic-centimeter-to-cubic-inch conversion factor"), "0.0610");
 
     for (i = 0; i < MAX_REGISTERS; i++) {
         mp_set_from_integer(0, v->MPmvals[i]);
@@ -747,14 +670,11 @@ main(int argc, char **argv)
     get_options(argc, argv);   /* Get command line arguments. */
     ui_init(&argc, &argv);     /* Initialise UI */
     resources_init();          /* Initialise configuration */
-    display_init();
+    display_init(&v->display);
 
-    init_text();               /* Setup text strings depending upon language. */
     read_resources();          /* Read resources from merged database. */
     ui_load();
 
-    do_clear();                /* Initialise and clear display. */
-
     ui_start();                /* Display the calculator. */
     
     return(0);
