@@ -53,43 +53,34 @@
      tCLR = 269,
      tCOS = 270,
      tCOSH = 271,
-     tCTRM = 272,
-     tDDB = 273,
-     tEXP = 274,
-     tFRAC = 275,
-     tFV = 276,
-     tINT = 277,
-     tLN = 278,
-     tLOG10 = 279,
-     tLOG2 = 280,
-     tMOD = 281,
-     tNOT = 282,
-     tOR = 283,
-     tPI = 284,
-     tPMT = 285,
-     tPV = 286,
-     tRAND = 287,
-     tRATE = 288,
-     tRCL = 289,
-     tSIN = 290,
-     tSINH = 291,
-     tSLN = 292,
-     tSQRT = 293,
-     tSTO = 294,
-     tSYD = 295,
-     tTAN = 296,
-     tTANH = 297,
-     tTERM = 298,
-     tU16 = 299,
-     tU32 = 300,
-     tXNOR = 301,
-     tXOR = 302,
-     tNUMBER = 303,
-     tREG = 304,
-     NEG = 305,
-     MED = 306,
-     POS = 307,
-     HIGH = 308
+     tEXP = 272,
+     tFRAC = 273,
+     tINT = 274,
+     tLN = 275,
+     tLOG10 = 276,
+     tLOG2 = 277,
+     tMOD = 278,
+     tNOT = 279,
+     tOR = 280,
+     tPI = 281,
+     tRAND = 282,
+     tRCL = 283,
+     tSIN = 284,
+     tSINH = 285,
+     tSQRT = 286,
+     tSTO = 287,
+     tTAN = 288,
+     tTANH = 289,
+     tU16 = 290,
+     tU32 = 291,
+     tXNOR = 292,
+     tXOR = 293,
+     tNUMBER = 294,
+     tREG = 295,
+     NEG = 296,
+     MED = 297,
+     POS = 298,
+     HIGH = 299
    };
 #endif
 /* Tokens.  */
@@ -107,43 +98,34 @@
 #define tCLR 269
 #define tCOS 270
 #define tCOSH 271
-#define tCTRM 272
-#define tDDB 273
-#define tEXP 274
-#define tFRAC 275
-#define tFV 276
-#define tINT 277
-#define tLN 278
-#define tLOG10 279
-#define tLOG2 280
-#define tMOD 281
-#define tNOT 282
-#define tOR 283
-#define tPI 284
-#define tPMT 285
-#define tPV 286
-#define tRAND 287
-#define tRATE 288
-#define tRCL 289
-#define tSIN 290
-#define tSINH 291
-#define tSLN 292
-#define tSQRT 293
-#define tSTO 294
-#define tSYD 295
-#define tTAN 296
-#define tTANH 297
-#define tTERM 298
-#define tU16 299
-#define tU32 300
-#define tXNOR 301
-#define tXOR 302
-#define tNUMBER 303
-#define tREG 304
-#define NEG 305
-#define MED 306
-#define POS 307
-#define HIGH 308
+#define tEXP 272
+#define tFRAC 273
+#define tINT 274
+#define tLN 275
+#define tLOG10 276
+#define tLOG2 277
+#define tMOD 278
+#define tNOT 279
+#define tOR 280
+#define tPI 281
+#define tRAND 282
+#define tRCL 283
+#define tSIN 284
+#define tSINH 285
+#define tSQRT 286
+#define tSTO 287
+#define tTAN 288
+#define tTANH 289
+#define tU16 290
+#define tU32 291
+#define tXNOR 292
+#define tXOR 293
+#define tNUMBER 294
+#define tREG 295
+#define NEG 296
+#define MED 297
+#define POS 298
+#define HIGH 299
 
 
 
@@ -156,7 +138,7 @@ typedef union YYSTYPE
   int integer;
 }
 /* Line 1489 of yacc.c.  */
-#line 160 "ce_parser.tab.h"
+#line 142 "ce_parser.tab.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
