@@ -20,6 +20,7 @@
 #include <assert.h>
 #include <errno.h>
 
+#include "mp.h"
 #include "mpmath.h"
 
 static char digits[] = "0123456789ABCDEF";
@@ -58,7 +59,7 @@ setbool(BOOLEAN p)
 
 
 void
-calc_and(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
+calc_and(const int s1[MP_SIZE], const int s2[MP_SIZE], int t[MP_SIZE])
 {
     double dres, dval;
 
@@ -70,7 +71,7 @@ calc_and(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
 
 
 void
-calc_or(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
+calc_or(const int s1[MP_SIZE], const int s2[MP_SIZE], int t[MP_SIZE])
 {
     double dres, dval;
 
@@ -82,7 +83,7 @@ calc_or(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
 
 
 void
-calc_xor(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
+calc_xor(const int s1[MP_SIZE], const int s2[MP_SIZE], int t[MP_SIZE])
 {
     double dres, dval;
 
@@ -94,7 +95,7 @@ calc_xor(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
 
 
 void
-calc_xnor(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
+calc_xnor(const int s1[MP_SIZE], const int s2[MP_SIZE], int t[MP_SIZE])
 {
     double dres, dval;
 
@@ -106,7 +107,7 @@ calc_xnor(int t[MP_SIZE], int s1[MP_SIZE], int s2[MP_SIZE])
 
 
 void
-calc_not(int s1[MP_SIZE], int t[MP_SIZE])
+calc_not(const int s1[MP_SIZE], int t[MP_SIZE])
 {
     double dval = mp_cast_to_double(s1);
     
@@ -123,7 +124,7 @@ calc_rand(int t[MP_SIZE])
 
 
 void
-calc_u32(int s1[MP_SIZE], int t1[MP_SIZE])
+calc_u32(const int s1[MP_SIZE], int t1[MP_SIZE])
 {
     double dval = mp_cast_to_double(s1);
     dval = setbool(ibool(dval));
@@ -132,35 +133,24 @@ calc_u32(int s1[MP_SIZE], int t1[MP_SIZE])
 
 
 void
-calc_u16(int s1[MP_SIZE], int t1[MP_SIZE])
+calc_u16(const int s1[MP_SIZE], int t1[MP_SIZE])
 {
     double dval = mp_cast_to_double(s1);
     dval = setbool(ibool(dval) & 0xffff);
     mp_set_from_double(dval, t1);
 }
 
-
 void
-calc_inv(int s1[MP_SIZE], int t1[MP_SIZE])     /* Calculate 1/x */
+calc_inv(const int s1[MP_SIZE], int t1[MP_SIZE])     /* Calculate 1/x */
 {
     int MP1[MP_SIZE];
     int MP2[MP_SIZE];
 
     mp_set_from_integer(1, MP1);
-    mpstr(s1, MP2);
+    mp_set_from_mp(s1, MP2);
     mpdiv(MP1, MP2, t1);
 }
 
-
-void 
-calc_tenpowx(int s1[MP_SIZE], int t1[MP_SIZE])   /* Calculate 10^x */
-{
-    int MP1[MP_SIZE];
-    mp_set_from_integer(10, MP1);
-    mppwr2(MP1, s1, t1);
-}
-
-
 void
 calc_xpowy(int MPx[MP_SIZE], int MPy[MP_SIZE], int MPres[MP_SIZE]) /* Do x^y */
 {
@@ -180,7 +170,7 @@ calc_xpowy(int MPx[MP_SIZE], int MPy[MP_SIZE], int MPres[MP_SIZE]) /* Do x^y */
         mpcmim(MPy, MPtmp);
         if (mp_is_equal(MPtmp, MPy)) {   /* Is y == int(y) ? */
             int y = mp_cast_to_int(MPy);
-            mppwr(MPx, &y, MPres);
+            mppwr(MPx, y, MPres);
         } else {        /* y != int(y). Force mppwr2 to generate an error. */
             mppwr2(MPx, MPy, MPres);
         }
@@ -189,16 +179,6 @@ calc_xpowy(int MPx[MP_SIZE], int MPy[MP_SIZE], int MPres[MP_SIZE]) /* Do x^y */
     }
 }
 
-
-void 
-calc_xtimestenpowx(int s1[MP_SIZE], int s2[MP_SIZE], int t1[MP_SIZE])
-{
-    int MP1[MP_SIZE];
-
-    calc_tenpowx(s2, MP1);
-    mpmul(s1, MP1, t1);
-}
-
 int
 calc_modulus(int op1[MP_SIZE], 
 	     int op2[MP_SIZE], 
@@ -213,13 +193,13 @@ calc_modulus(int op1[MP_SIZE],
     mpdiv(op1, op2, MP1);
     mpcmim(MP1, MP1);
     mpmul(MP1, op2, MP2);
-    mpsub(op1, MP2, result);
+    mp_subtract(op1, MP2, result);
 
     mp_set_from_integer(0, MP1);
     if ((mp_is_less_than(op2, MP1)
 	 && mp_is_greater_than(result, MP1)) ||
 	mp_is_less_than(result, MP1)) { 
-        mpadd(result, op2, result);
+        mp_add(result, op2, result);
     }
 
     return 0;
@@ -230,7 +210,7 @@ calc_percent(int s1[MP_SIZE], int t1[MP_SIZE])
 {
     int MP1[MP_SIZE];
 
-    MPstr_to_num("0.01", DEC, MP1);
+    MPstr_to_num("0.01", 10, MP1);
     mpmul(s1, MP1, t1);
 }
 
@@ -241,23 +221,6 @@ do_e(int t1[MP_SIZE])
 }
 
 
-static void 
-mptan(int s1[MP_SIZE], int t1[MP_SIZE])
-{
-    int MPcos[MP_SIZE]; 
-    int MPsin[MP_SIZE];
-    double cval;
-
-    mpsin(s1, MPsin);
-    mpcos(s1, MPcos);
-    cval = mp_cast_to_double(MPcos);
-    if (cval == 0.0) {
-        doerr(_("Error, cannot calculate cosine"));
-    }
-    mpdiv(MPsin, MPcos, t1);
-}
-
-
 /* Change type to radian */
 
 static void
@@ -266,17 +229,17 @@ to_rad(int s1[MP_SIZE], int t1[MP_SIZE])
     int MP1[MP_SIZE], MP2[MP_SIZE];
 
     if (v->ttype == DEG) {
-        mppi(MP1);
+        mp_get_pi(MP1);
         mpmul(s1, MP1, MP2);
         mp_set_from_integer(180, MP1);
         mpdiv(MP2, MP1, t1);
     } else if (v->ttype == GRAD) {
-        mppi(MP1);
+        mp_get_pi(MP1);
         mpmul(s1, MP1, MP2);
         mp_set_from_integer(200, MP1);
         mpdiv(MP2, MP1, t1);
     } else {
-        mpstr(s1, t1);
+        mp_set_from_mp(s1, t1);
     }
 }
 
@@ -291,18 +254,18 @@ do_trig_typeconv(enum trig_type ttype, int s1[MP_SIZE], int t1[MP_SIZE])
         case DEG:
             mp_set_from_integer(180, MP1);
             mpmul(s1, MP1, MP2);
-            mppi(MP1);
+            mp_get_pi(MP1);
             mpdiv(MP2, MP1, t1);
             break;
 
         case RAD:
-            mpstr(s1, t1);
+            mp_set_from_mp(s1, t1);
             break;
 
         case GRAD:
             mp_set_from_integer(200, MP1);
             mpmul(s1, MP1, MP2);
-            mppi(MP1);
+            mp_get_pi(MP1);
             mpdiv(MP2, MP1, t1);
             break;
 
@@ -312,139 +275,6 @@ do_trig_typeconv(enum trig_type ttype, int s1[MP_SIZE], int t1[MP_SIZE])
     }
 }
 
-
-/*  The following MP routines were not in the Brent FORTRAN package. They are
- *  derived here, in terms of the existing routines.
- */
-
-/*  MP precision arc cosine.
- *
- *  1. If (x < -1.0  or x > 1.0) then report DOMAIN error and return 0.0.
- *
- *  2. If (x = 0.0) then acos(x) = PI/2.
- *
- *  3. If (x = 1.0) then acos(x) = 0.0
- *
- *  4. If (x = -1.0) then acos(x) = PI.
- *
- *  5. If (0.0 < x < 1.0) then  acos(x) = atan(sqrt(1-(x**2)) / x)
- *
- *  6. If (-1.0 < x < 0.0) then acos(x) = atan(sqrt(1-(x**2)) / x) + PI
- */
-
-static void
-mpacos(int *MPx, int *MPretval)
-{
-    int MP0[MP_SIZE],  MP1[MP_SIZE],  MP2[MP_SIZE];
-    int MPn1[MP_SIZE], MPpi[MP_SIZE], MPy[MP_SIZE], val;
-
-    mppi(MPpi);
-    mp_set_from_integer(0, MP0);
-    mp_set_from_integer(1, MP1);
-    mp_set_from_integer(-1, MPn1);
-
-    if (mp_is_greater_than(MPx, MP1) || mp_is_less_than(MPx, MPn1)) {
-        doerr(_("Error"));
-        mpstr(MP0, MPretval);
-    } else if (mp_is_equal(MPx, MP0)) {
-        val = 2;
-        mpdivi(MPpi, &val, MPretval);
-    } else if (mp_is_equal(MPx, MP1)) {
-        mpstr(MP0, MPretval);
-    } else if (mp_is_equal(MPx, MPn1)) {
-        mpstr(MPpi, MPretval);
-    } else { 
-        mpmul(MPx, MPx, MP2);
-        mpsub(MP1, MP2, MP2);
-        mpsqrt(MP2, MP2);
-        mpdiv(MP2, MPx, MP2);
-        mpatan(MP2, MPy);
-        if (mp_is_greater_than(MPx, MP0)) {
-            mpstr(MPy, MPretval);
-        } else {
-            mpadd(MPy, MPpi, MPretval);
-        }
-    }
-}
-
-
-/*  MP precision hyperbolic arc cosine.
- *
- *  1. If (x < 1.0) then report DOMAIN error and return 0.0.
- *
- *  2. acosh(x) = log(x + sqrt(x**2 - 1))
- */
-
-static void
-mpacosh(int *MPx, int *MPretval)
-{
-    int MP1[MP_SIZE];
-
-    mp_set_from_integer(1, MP1);
-    if (mp_is_less_than(MPx, MP1)) {
-        doerr(_("Error"));
-        mp_set_from_integer(0, MPretval);
-    } else {
-        int val = -1;
-        mpmul(MPx, MPx, MP1);
-        mpaddi(MP1, &val, MP1);
-        mpsqrt(MP1, MP1);
-        mpadd(MPx, MP1, MP1);
-        mpln(MP1, MPretval);
-    }
-}
-
-
-/*  MP precision hyperbolic arc sine.
- *
- *  1. asinh(x) = log(x + sqrt(x**2 + 1))
- */
-
-static void
-mpasinh(int *MPx, int *MPretval)
-{
-    int MP1[MP_SIZE], val;
- 
-    mpmul(MPx, MPx, MP1);
-    val = 1;
-    mpaddi(MP1, &val, MP1);
-    mpsqrt(MP1, MP1);
-    mpadd(MPx, MP1, MP1);
-    mpln(MP1, MPretval);
-}
-
-
-/*  MP precision hyperbolic arc tangent.
- *
- *  1. If (x <= -1.0 or x >= 1.0) then report a DOMAIn error and return 0.0.
- *
- *  2. atanh(x) = 0.5 * log((1 + x) / (1 - x))
- */
-
-static void
-mpatanh(int *MPx, int *MPretval)
-{
-    int MP0[MP_SIZE], MP1[MP_SIZE], MP2[MP_SIZE];
-    int MP3[MP_SIZE], MPn1[MP_SIZE];
-
-    mp_set_from_integer(0, MP0);
-    mp_set_from_integer(1, MP1);
-    mp_set_from_integer(-1, MPn1);
-
-    if (mp_is_greater_equal(MPx, MP1) || mp_is_less_equal(MPx, MPn1)) {
-        doerr(_("Error"));
-        mpstr(MP0, MPretval);
-    } else {
-        mpadd(MP1, MPx, MP2);
-        mpsub(MP1, MPx, MP3);
-        mpdiv(MP2, MP3, MP3);
-        mpln(MP3, MP3);
-        MPstr_to_num("0.5", DEC, MP1);
-        mpmul(MP1, MP3, MPretval);
-    }
-}
-
-
 /*  MP precision common log.
  *
  *  1. log10(x) = log10(e) * log(x)
@@ -461,242 +291,6 @@ mplogn(int n, int *MPx, int *MPretval)
     mpdiv(MP2, MP1, MPretval);
 }
 
-
-void
-calc_ctrm(int t[MP_SIZE])
-{
-
-/*  Cterm - MEM0 = int (periodic interest rate).
- *          MEM1 = fv  (future value).
- *          MEM2 = pv  (present value).
- *
- *          RESULT = log(MEM1 / MEM2) / log(1 + MEM0)
- */
-
-    int val;
-    int MP1[MP_SIZE], MP2[MP_SIZE], MP3[MP_SIZE], MP4[MP_SIZE];
-
-    mpdiv(v->MPmvals[1], v->MPmvals[2], MP1);
-    mpln(MP1, MP2);
-    val = 1;
-    mpaddi(v->MPmvals[0], &val, MP3);
-    mpln(MP3, MP4);
-    mpdiv(MP2, MP4, t);
-}
-
-
-void
-calc_ddb(int t[MP_SIZE])
-{
-
-/*  Ddb   - MEM0 = cost    (amount paid for asset).
- *          MEM1 = salvage (value of asset at end of its life).
- *          MEM2 = life    (useful life of the asset).
- *          MEM3 = period  (time period for depreciation allowance).
- *
- *          bv = 0.0;
- *          for (i = 0; i < MEM3; i++)
- *            {
- *              VAL = ((MEM0 - bv) * 2) / MEM2
- *              bv += VAL
- *            }
- *          RESULT = VAL
- */
-
-    int i;
-    int len;
-    int val;
-    int MPbv[MP_SIZE], MP1[MP_SIZE], MP2[MP_SIZE];
-
-    mp_set_from_integer(0, MPbv);
-    len = mp_cast_to_int(v->MPmvals[3]);
-    for (i = 0; i < len; i++) {
-        mpsub(v->MPmvals[0], MPbv, MP1);
-        val = 2;
-        mpmuli(MP1, &val, MP2);
-        mpdiv(MP2, v->MPmvals[2], t);
-        mpstr(MPbv, MP1);
-        mpadd(MP1, t, MPbv); /* TODO: why result is MPbv, for next loop? */
-    }
-}
-
-
-void
-calc_fv(int t[MP_SIZE])
-{
-
-/*  Fv    - MEM0 = pmt (periodic payment).
- *          MEM1 = int (periodic interest rate).
- *          MEM2 = n   (number of periods).
- *
- *          RESULT = MEM0 * (pow(1 + MEM1, MEM2) - 1) / MEM1
- */
-
-    int val;
-    int MP1[MP_SIZE], MP2[MP_SIZE], MP3[MP_SIZE], MP4[MP_SIZE];
-  
-    val = 1;
-    mpaddi(v->MPmvals[1], &val, MP1);
-    mppwr2(MP1, v->MPmvals[2], MP2);
-    val = -1;
-    mpaddi(MP2, &val, MP3);
-    mpmul(v->MPmvals[0], MP3, MP4);
-    mpdiv(MP4, v->MPmvals[1], t);
-}
-
-
-void
-calc_pmt(int t[MP_SIZE])
-{
-
-/*  Pmt   - MEM0 = prin (principal).
- *          MEM1 = int  (periodic interest rate).
- *          MEM2 = n    (term).
- *
- *          RESULT = MEM0 * (MEM1 / (1 - pow(MEM1 + 1, -1 * MEM2)))
- */
-
-    int val;
-    int MP1[MP_SIZE], MP2[MP_SIZE], MP3[MP_SIZE], MP4[MP_SIZE];
-
-    val = 1;
-    mpaddi(v->MPmvals[1], &val, MP1);
-    val = -1;
-    mpmuli(v->MPmvals[2], &val, MP2);
-    mppwr2(MP1, MP2, MP3);
-    val = -1;
-    mpmuli(MP3, &val, MP4);
-    val = 1;
-    mpaddi(MP4, &val, MP1);
-    mpdiv(v->MPmvals[1], MP1, MP2);
-    mpmul(v->MPmvals[0], MP2, t);
-}
-
-
-void
-calc_pv(int t[MP_SIZE])
-{
-
-/*  Pv    - MEM0 = pmt (periodic payment).
- *          MEM1 = int (periodic interest rate).
- *          MEM2 = n   (term).
- *
- *          RESULT = MEM0 * (1 - pow(1 + MEM1, -1 * MEM2)) / MEM1
- */
-
-    int val;
-    int MP1[MP_SIZE], MP2[MP_SIZE], MP3[MP_SIZE], MP4[MP_SIZE];
-
-    val = 1;
-    mpaddi(v->MPmvals[1], &val, MP1);
-    val = -1;
-    mpmuli(v->MPmvals[2], &val, MP2);
-    mppwr2(MP1, MP2, MP3);
-    val = -1;
-    mpmuli(MP3, &val, MP4);
-    val = 1;
-    mpaddi(MP4, &val, MP1);
-    mpdiv(MP1, v->MPmvals[1], MP2);
-    mpmul(v->MPmvals[0], MP2, t);
-}
-
-
-void
-calc_rate(int t[MP_SIZE])
-{
-
-/*  Rate  - MEM0 = fv (future value).
- *          MEM1 = pv (present value).
- *          MEM2 = n  (term).
- *
- *          RESULT = pow(MEM0 / MEM1, 1 / MEM2) - 1
- */
-
-    int val;
-    int MP1[MP_SIZE], MP2[MP_SIZE], MP3[MP_SIZE], MP4[MP_SIZE];
-
-    mpdiv(v->MPmvals[0], v->MPmvals[1], MP1);
-    mp_set_from_integer(1, MP2);
-    mpdiv(MP2, v->MPmvals[2], MP3);
-    mppwr2(MP1, MP3, MP4);
-    val = -1;
-    mpaddi(MP4, &val, t);
-}
-
-
-void
-calc_sln(int t[MP_SIZE])
-{
-
-/*  Sln   - MEM0 = cost    (cost of the asset).
- *          MEM1 = salvage (salvage value of the asset).
- *          MEM2 = life    (useful life of the asset).
- *
- *          RESULT = (MEM0 - MEM1) / MEM2
- */
-  
-    int MP1[MP_SIZE];
-
-    mpsub(v->MPmvals[0], v->MPmvals[1], MP1);
-    mpdiv(MP1, v->MPmvals[2], t);
-}
-
-
-void
-calc_syd(int t[MP_SIZE])
-{
-
-/*  Syd   - MEM0 = cost    (cost of the asset).
- *          MEM1 = salvage (salvage value of the asset).
- *          MEM2 = life    (useful life of the asset).
- *          MEM3 = period  (period for which depreciation is computed).
- *
- *          RESULT = ((MEM0 - MEM1) * (MEM2 - MEM3 + 1)) /
- *                   (MEM2 * (MEM2 + 1) / 2)
- */
-
-    int val;
-    int MP1[MP_SIZE], MP2[MP_SIZE], MP3[MP_SIZE], MP4[MP_SIZE];
-
-    mpsub(v->MPmvals[2], v->MPmvals[3], MP2);
-    val = 1;
-    mpaddi(MP2, &val, MP3);
-    mpaddi(v->MPmvals[2], &val, MP2);
-    mpmul(v->MPmvals[2], MP2, MP4);
-    mp_set_from_integer(2, MP2);
-    mpdiv(MP4, MP2, MP1);
-    mpdiv(MP3, MP1, MP2);
-    mpsub(v->MPmvals[0], v->MPmvals[1], MP1);
-    mpmul(MP1, MP2, t);
-}
-
-
-void
-calc_term(int t[MP_SIZE])
-{
-
-/*  Term  - MEM0 = pmt (periodic payment).
- *          MEM1 = fv  (future value).
- *          MEM2 = int (periodic interest rate).
- *
- *          RESULT = log(1 + (MEM1 * MEM2 / MEM0)) / log(1 + MEM2)
- */
-
-    int val;
-    int MP1[MP_SIZE], MP2[MP_SIZE], MP3[MP_SIZE], MP4[MP_SIZE];
-
-    val = 1;
-    mpaddi(v->MPmvals[2], &val, MP1);
-    mpln(MP1, MP2);
-    mpmul(v->MPmvals[1], v->MPmvals[2], MP1);
-    mpdiv(MP1, v->MPmvals[0], MP3);
-    val = 1;
-    mpaddi(MP3, &val, MP4);
-    mpln(MP4, MP1);
-    mpdiv(MP1, MP2, t);
-}
-
-
 void
 calc_shift(int s[MP_SIZE], int t[MP_SIZE], int times)
 {
@@ -756,7 +350,7 @@ is_natural(int MPnum[MP_SIZE])
     if (!is_integer(MPnum)) {
         return 0;
     }
-    mpabs(MPnum, MP1);
+    mp_abs(MPnum, MP1);
     return mp_is_equal(MPnum, MP1);
 }
 
@@ -765,7 +359,7 @@ calc_epowy(int s[MP_SIZE], int t[MP_SIZE])
 {
     int MP1[MP_SIZE];
     
-    mpstr(s, MP1);
+    mp_set_from_mp(s, MP1);
     mpexp(MP1, t);
 }
 
@@ -777,56 +371,56 @@ calc_trigfunc(enum trigfunc_type type, int s1[MP_SIZE], int t1[MP_SIZE])
     switch (type) {
         case sin_t: 
             to_rad(s1, s1);
-            mpsin(s1, t1);
+            mp_sin(s1, t1);
             break;
 
         case cos_t:
             to_rad(s1, s1);
-            mpcos(s1, t1);
+            mp_cos(s1, t1);
             break;
 
         case tan_t:
             to_rad(s1, s1);
-            mptan(s1, t1);
+            mp_tan(s1, t1);
             break;
 
         case sinh_t:
-            mpsinh(s1, t1);
+            mp_sinh(s1, t1);
             break;
 
         case cosh_t:
-            mpcosh(s1, t1);
+            mp_cosh(s1, t1);
             break;
 
         case tanh_t:
-            mptanh(s1, t1);
+            mp_tanh(s1, t1);
             break;
 
         case asin_t:
-            mpasin(s1, t1);
+            mp_asin(s1, t1);
             do_trig_typeconv(v->ttype, t1, t1);
             break;
 
         case acos_t:
-            mpacos(s1, t1);
+            mp_acos(s1, t1);
             do_trig_typeconv(v->ttype, t1, t1);
             break;
 
         case atan_t:
-            mpatan(s1, t1);
+            mp_atan(s1, t1);
             do_trig_typeconv(v->ttype, t1, t1);
             break;
 
         case asinh_t:
-            mpasinh(s1, t1);
+            mp_asinh(s1, t1);
             break;
 
         case acosh_t:
-            mpacosh(s1, t1);
+            mp_acosh(s1, t1);
             break;
 
         case atanh_t:
-            mpatanh(s1, t1);
+            mp_atanh(s1, t1);
             break;
     }
 
@@ -847,7 +441,7 @@ make_fixed(char *target, int target_len, int *MPnumber, int base, int cmax, int
     int dval, i;
  
     optr = target;
-    mpabs(MPnumber, MPval);
+    mp_abs(MPnumber, MPval);
     mp_set_from_integer(0, MP1);
     if (mp_is_less_than(MPnumber, MP1)) {
         *optr++ = '-';
@@ -855,12 +449,12 @@ make_fixed(char *target, int target_len, int *MPnumber, int base, int cmax, int
 
     mp_set_from_integer(basevals[base], MP1base);
 
-    mppwr(MP1base, &v->accuracy, MP1);
+    mppwr(MP1base, v->accuracy, MP1);
     /* FIXME: string const. if MPstr_to_num can get it */
     SNPRINTF(half, MAXLINE, "0.5");
-    MPstr_to_num(half, DEC, MP2);
+    MPstr_to_num(half, 10, MP2);
     mpdiv(MP2, MP1, MP1);
-    mpadd(MPval, MP1, MPval);
+    mp_add(MPval, MP1, MPval);
 
     mp_set_from_integer(1, MP2);
     if (mp_is_less_than(MPval, MP2)) {
@@ -889,13 +483,9 @@ make_fixed(char *target, int target_len, int *MPnumber, int base, int cmax, int
 
         *optr++ = digits[dval];
         dval = -dval;
-        mpaddi(MPval, &dval, MPval);
+        mp_add_integer(MPval, dval, MPval);
     }    
     *optr++ = '\0';
-    if (toclear == TRUE) {
-        v->ltr.toclear = 1;
-    }
-    v->ltr.pointed = 0;
 
     /* Strip off trailing zeroes */
     if (!v->show_zeroes) {
@@ -919,7 +509,7 @@ make_eng_sci(char *target, int target_len, int *MPnumber, int base)
     char half[MAXLINE], fixed[MAX_DIGITS], *optr;
     int MP1[MP_SIZE], MPatmp[MP_SIZE], MPval[MP_SIZE];
     int MP1base[MP_SIZE], MP3base[MP_SIZE], MP10base[MP_SIZE];
-    int i, dval, len, n;
+    int i, dval, len;
     int MPmant[MP_SIZE];        /* Mantissa. */
     int ddig;                   /* Number of digits in exponent. */
     int eng = 0;                /* Set if this is an engineering number. */
@@ -929,19 +519,17 @@ make_eng_sci(char *target, int target_len, int *MPnumber, int base)
         eng = 1;
     }
     optr = target;
-    mpabs(MPnumber, MPval);
+    mp_abs(MPnumber, MPval);
     mp_set_from_integer(0, MP1);
     if (mp_is_less_than(MPnumber, MP1)) {
         *optr++ = '-';
     }
-    mpstr(MPval, MPmant);
+    mp_set_from_mp(MPval, MPmant);
 
     mp_set_from_integer(basevals[base], MP1base);
-    n = 3;
-    mppwr(MP1base, &n, MP3base);
+    mppwr(MP1base, 3, MP3base);
 
-    n = 10;
-    mppwr(MP1base, &n, MP10base);
+    mppwr(MP1base, 10, MP10base);
 
     mp_set_from_integer(1, MP1);
     mpdiv(MP1, MP10base, MPatmp);
@@ -987,8 +575,8 @@ make_eng_sci(char *target, int target_len, int *MPnumber, int base)
     }
  
     SNPRINTF(half, MAXLINE, "0.5");
-    MPstr_to_num(half, DEC, MP1);
-    mpaddi(MP1, &exp, MPval);
+    MPstr_to_num(half, 10, MP1);
+    mp_add_integer(MP1, exp, MPval);
     mp_set_from_integer(1, MP1);
     for (ddig = 0; mp_is_greater_equal(MPval, MP1); ddig++) {
         mpdiv(MPval, MP1base, MPval);
@@ -1003,11 +591,9 @@ make_eng_sci(char *target, int target_len, int *MPnumber, int base)
         dval = mp_cast_to_int(MPval);
         *optr++ = digits[dval];
         dval = -dval;
-        mpaddi(MPval, &dval, MPval);
+        mp_add_integer(MPval, dval, MPval);
     }
     *optr++    = '\0';
-    v->ltr.toclear = 1;
-    v->ltr.pointed = 0;
 }
 
 
@@ -1042,90 +628,48 @@ make_number(char *target, int target_len, int *MPnumber, int base, int ignoreErr
     }
 }
 
-
-static int
-char_val(char chr)
+/* Calculate the factorial of MPval. */
+void
+calc_factorial(int *MPval, int *MPres)
 {
-    if (chr >= '0' && chr <= '9') {
-        return(chr - '0');
-    } else if (chr >= 'a' && chr <= 'f') {
-        return(chr - 'a' + 10);
-    } else if (chr >= 'A' && chr <= 'F') {
-        return(chr - 'A' + 10);
-    } else {
-        return(-1);
-    }
-}
+    double val;
+    int i, MPa[MP_SIZE], MP1[MP_SIZE], MP2[MP_SIZE];
 
-
-/* Convert string into an MP number, in the given base
+/*  NOTE: do_factorial, on each iteration of the loop, will attempt to
+ *        convert the current result to a double. If v->error is set,
+ *        then we've overflowed. This is to provide the same look&feel
+ *        as V3.
+ *
+ *  XXX:  Needs to be improved. Shouldn't need to convert to a double in
+ *        order to check this.
  */
 
-void
-MPstr_to_num(char *str, enum base_type base, int *MPval)
-{
-    char *optr;
-    int MP1[MP_SIZE], MP2[MP_SIZE], MPbase[MP_SIZE];
-    int i, inum;
-    int exp      = 0;
-    int exp_sign = 1;
-    int negate = 0;
-
-    mp_set_from_integer(0, MPval);
-    mp_set_from_integer(basevals[(int) base], MPbase);
-
-    optr = str;
-
-    /* Remove any initial spaces or tabs. */
-    while (*optr == ' ' || *optr == '\t') {
-        optr++;
-    }
-
-    /* Check if this is a negative number. */
-    if (*optr == '-') {
-        negate = 1;
-        optr++;
-    }
-
-    while ((inum = char_val(*optr)) >= 0) {
-        mpmul(MPval, MPbase, MPval);
-        mpaddi(MPval, &inum, MPval);
-        optr++;
-    }
-
-    if (*optr == '.' || *optr == *v->radix) {
-        optr++;
-        for (i = 1; (inum = char_val(*optr)) >= 0; i++) {
-            mppwr(MPbase, &i, MP1);
-            mp_set_from_integer(inum, MP2);
-            mpdiv(MP2, MP1, MP1);
-            mpadd(MPval, MP1, MPval);
-        optr++;
+    mp_set_from_mp(MPval, MPa);
+    mpcmim(MPval, MP1);
+    mp_set_from_integer(0, MP2);
+    if (mp_is_equal(MPval, MP1)
+	&& mp_is_greater_equal(MPval, MP2)) {   /* Only positive integers. */
+        if (mp_is_equal(MP1, MP2)) {    /* Special case for 0! */
+            mp_set_from_integer(1, MPres);
+            return;
         }
-    }
-
-    while (*optr == ' ') {
-        optr++;
-    }
- 
-    if (*optr != '\0') {
-        if (*optr == '-') {
-            exp_sign = -1;
-        }
- 
-        while ((inum = char_val(*++optr)) >= 0) {
-            exp = exp * basevals[(int) base] + inum;
+        mp_set_from_integer(1, MPa);
+        i = mp_cast_to_int(MP1);
+        if (!i) {
+            matherr((struct exception *) NULL);
+        } else {
+            while (i > 0) {
+                mpmuli(MPa, i, MPa);
+                val = mp_cast_to_double(MPa);
+                if (v->error) {
+                    mperr("Error calculating factorial\n");
+                    return;
+                }
+                i--;
+            }
         }
+    } else {
+        matherr((struct exception *) NULL);
     }
-    exp *= exp_sign;
-
-    if (v->ltr.key_exp) {
-        mppwr(MPbase, &exp, MP1);
-        mpmul(MPval, MP1, MPval);
-    }
-
-    if (negate == 1) {
-        mpneg(MPval, MPval);
-    }
+    mp_set_from_mp(MPa, MPres);
 }
-
